const path = require('path')
const webpack = require('webpack')
const HappyPack = require('happypack')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin')
const HtmlWebpackIncludeAssetsPlugin = require('html-webpack-include-assets-plugin')
const ScriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin')
const ResourceHintWebpackPlugin = require('resource-hints-webpack-plugin')
const PreloadWebpackPlugin = require('preload-webpack-plugin')
const { projectPath, resolve } = require('../utils')
const appConfig = require('../utils/get-app-config')
const vueOptions = require('./vue-loader.conf')
const styleOptions = require('./style-loader')
// 如果用了html-webpack-plugin则不需要
// const VueSSRClientPlugin = require('vue-server-renderer/client-plugin')

let cssIncludes = [
  resolve('client'),
  resolve('node_modules/normalize.css'),
]

if (appConfig.webpack.cssInclude && appConfig.webpack.cssInclude.length) {
  cssIncludes = cssIncludes.concat(appConfig.webpack.cssInclude)
}

const imagePath = appConfig.image && appConfig.image.path || 'images'

const config = {
  entry: appConfig.webpack.entry,
  mode: 'none',
  output: {
    filename: '[name].js',
    path: `${projectPath}/public`,
  },
  module: {
    rules: [
      {
        test: /\.jsx?$/,
        loader: 'happypack/loader?id=babel',
        include: [
          resolve('client'),
          resolve('node_modules/summerd-sdk/client'),
        ],
      },
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        include: resolve('client'),
        options: vueOptions,
      },
      {
        test: /\.css$/,
        loader: styleOptions.cssLoader({}).css,
        include: cssIncludes,
      },
      {
        test: /\.styl$/,
        loader: styleOptions.cssLoader({}).stylus,
        include: cssIncludes,
      },
      {
        test: /\.webp$/,
        loaders: [
          {
            loader: 'file-loader',
            options: {
              name: `${imagePath}/webp/[name].[ext]?[hash]`,
            },
          },
        ],
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        // {
        //   loader: multi(
        //     'file-loader?name=images/webp/[name].webp!webp-loader?{quality: 60}',
        //     'file-loader?name=images/[name].[ext]'
        //   ),
        // },
        loaders: [
          {
            loader: 'url-loader',
            options: {
              limit: 8192,
              name: `${imagePath}/[name].[ext]?[hash]`,
            },
          },
        ],
      },
      {
        test: /\.(ico)$/,
        loaders: [
          {
            loader: 'url-loader',
            options: {
              name: 'static/[name].[ext]',
            },
          },
        ],
      },
      {
        test: /\.(woff2?|ttf|eot|otf)$/,
        loaders: [
          {
            loader: 'file-loader',
            options: {
              name: 'fonts/[name].[ext]?[hash]',
            },
          },
        ],
      },
    ],
  },
  resolve: {
    alias: {
      'vue$': 'vue/dist/vue.esm.js',
      '@': resolve('client'),
    }
  },
  plugins: [
    new HtmlWebpackPlugin({
      // favicon: path.join(projectPath, 'static', 'favicon.ico'),
      filename: 'index.html',
      template: path.join(projectPath, 'index.template.ejs'),
      inject: true,
      prefetch: ['**/*.*'],
      preload: ['**/*.*'],
    }),
    new ResourceHintWebpackPlugin(),
    // 必须放在HtmlWebpackPlugin之后
    new PreloadWebpackPlugin({
      rel: 'preload',
      as(entry) {
        if (/\.css$/.test(entry)) return 'style'
        if (/\.woff$/.test(entry)) return 'font'
        if (/\.png$/.test(entry)) return 'image'
        return 'script'
      }
    }),
    // 必须放在HtmlWebpackPlugin之后
    new ScriptExtHtmlWebpackPlugin({
      defaultAttribute: 'defer',
    }),
    new webpack.DllReferencePlugin({
      manifest: require(`${projectPath}/vendor_manifest.json`),
    }),
    new CopyWebpackPlugin([
      {
        from: `${projectPath}/static`,
        to: 'static/',
      },
    ]),
    new HtmlWebpackIncludeAssetsPlugin({
      assets: ['static/vendor.dll.js'],
      append: false,
    }),
    new HappyPack({
      id: 'babel',
      threads: 4,
      loaders: ['babel-loader?cacheDirectory=true'],
    }),
    // new VueSSRClientPlugin(),
  ],
}

if (appConfig.webpack.rules instanceof Array && appConfig.webpack.rules.length) {
  // css加载会报错，貌似没有添加上，这里加个处理
  config.module.rules.push(...appConfig.webpack.rules)
}

if (appConfig.webpack.plugins instanceof Array && appConfig.webpack.plugins.length) {
  config.plugins.push(...appConfig.webpack.plugins)
}

module.exports = config
